#!/usr/bin/env python3

import subprocess
import time

class command():
    def __init__(self):
        pass
    
    def initialize(self, environment):
        self.env = environment
        self.testMessage = "This is a voice test. If you can hear this message clearly, press Enter to accept this voice. Otherwise, wait 30 seconds to cancel."
    
    def shutdown(self):
        pass
    
    def getDescription(self):
        return "Configure speech module and voice with testing"
    
    def run(self):
        self.env['runtime']['outputManager'].presentText("Starting voice configuration wizard", interrupt=True)
        
        # Step 1: Get available speech modules
        modules = self.getSpeechdModules()
        if not modules:
            self.env['runtime']['outputManager'].presentText("No speech-dispatcher modules found. Please install speech-dispatcher.", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Error')
            return
        
        # For this implementation, cycle through modules or use the first available one
        # Get current module
        currentModule = self.env['runtime']['settingsManager'].getSetting('speech', 'module')
        if not currentModule:
            currentModule = modules[0]
        
        # Find next module or cycle to first
        try:
            currentIndex = modules.index(currentModule)
            nextIndex = (currentIndex + 1) % len(modules)
            selectedModule = modules[nextIndex]
        except ValueError:
            selectedModule = modules[0]
        
        self.env['runtime']['outputManager'].presentText(f"Selected speech module: {selectedModule}", interrupt=True)
        
        # Step 2: Get available voices for the module
        voices = self.getModuleVoices(selectedModule)
        if not voices:
            self.env['runtime']['outputManager'].presentText(f"No voices found for module {selectedModule}", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Error')
            return
        
        # Get current voice and cycle to next, or use best voice
        currentVoice = self.env['runtime']['settingsManager'].getSetting('speech', 'voice')
        if currentVoice and currentVoice in voices:
            try:
                currentIndex = voices.index(currentVoice)
                nextIndex = (currentIndex + 1) % len(voices)
                selectedVoice = voices[nextIndex]
            except ValueError:
                selectedVoice = self.selectBestVoice(voices)
        else:
            selectedVoice = self.selectBestVoice(voices)
        
        self.env['runtime']['outputManager'].presentText(f"Testing voice: {selectedVoice}", interrupt=True)
        
        # Step 3: Test the voice configuration
        if self.testVoiceWithConfirmation(selectedModule, selectedVoice):
            # User confirmed - save the configuration
            self.env['runtime']['settingsManager'].setSetting('speech', 'driver', 'speechdDriver')
            self.env['runtime']['settingsManager'].setSetting('speech', 'module', selectedModule)
            self.env['runtime']['settingsManager'].setSetting('speech', 'voice', selectedVoice)
            
            self.env['runtime']['outputManager'].presentText("Voice configuration saved successfully!", interrupt=True)
            self.env['runtime']['outputManager'].presentText(f"Module: {selectedModule}, Voice: {selectedVoice}", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Accept')
        else:
            # User cancelled or test failed
            self.env['runtime']['outputManager'].presentText("Voice configuration cancelled. Settings unchanged.", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Cancel')
    
    def getSpeechdModules(self):
        """Get list of available speech-dispatcher modules"""
        try:
            result = subprocess.run(['spd-say', '-O'], capture_output=True, text=True, timeout=10)
            if result.returncode == 0:
                lines = result.stdout.strip().split('\n')
                return [line.strip() for line in lines[1:] if line.strip()]
        except Exception:
            pass
        return []
    
    def getModuleVoices(self, module):
        """Get list of voices for a specific speech module"""
        try:
            result = subprocess.run(['spd-say', '-o', module, '-L'], capture_output=True, text=True, timeout=10)
            if result.returncode == 0:
                lines = result.stdout.strip().split('\n')
                voices = []
                for line in lines[1:]:
                    if not line.strip():
                        continue
                    if module.lower() == 'espeak-ng':
                        voice = self.processEspeakVoice(line)
                        if voice:
                            voices.append(voice)
                    else:
                        voices.append(line.strip())
                return voices
        except Exception:
            pass
        return []
    
    def processEspeakVoice(self, voiceLine):
        """Process espeak-ng voice line format"""
        parts = [p for p in voiceLine.split() if p]
        if len(parts) < 2:
            return None
        langCode = parts[-2].lower()
        variant = parts[-1].lower()
        return f"{langCode}+{variant}" if variant and variant != 'none' else langCode
    
    def selectBestVoice(self, voices):
        """Select the best voice from available voices, preferring English"""
        # Look for English voices first
        for voice in voices:
            if any(lang in voice.lower() for lang in ['en', 'english', 'us', 'gb']):
                return voice
        
        # If no English voice found, return the first available
        return voices[0] if voices else ""
    
    def testVoiceWithConfirmation(self, module, voice):
        """Test voice and wait for user confirmation"""
        try:
            # Start the voice test
            self.env['runtime']['outputManager'].presentText("Starting voice test. Listen carefully.", interrupt=True)
            time.sleep(1)  # Brief pause
            
            # Use spd-say to test the voice
            process = subprocess.Popen([
                'spd-say', '-o', module, '-y', voice, self.testMessage
            ])
            
            # Wait for the test message to finish (give it some time)
            time.sleep(2)
            
            # Now wait for user input
            self.env['runtime']['outputManager'].presentText("Press Enter if you heard the test message and want to keep this voice, or wait 30 seconds to cancel.", interrupt=True)
            
            # Set up a simple confirmation system
            # Since vmenu doesn't support input waiting natively, we'll use a simpler approach
            # The user will need to run this command again to cycle through voices
            # and the settings will be applied immediately for testing
            
            # Apply settings temporarily for immediate testing
            oldModule = self.env['runtime']['settingsManager'].getSetting('speech', 'module')
            oldVoice = self.env['runtime']['settingsManager'].getSetting('speech', 'voice')
            
            self.env['runtime']['settingsManager'].setSetting('speech', 'module', module)
            self.env['runtime']['settingsManager'].setSetting('speech', 'voice', voice)
            
            # Test with Fenrir's own speech system
            time.sleep(1)
            self.env['runtime']['outputManager'].presentText("This is how the new voice sounds in Fenrir. Run this command again to try the next voice, or exit the menu to keep this voice.", interrupt=True)
            
            # For now, we'll auto-accept since we can't wait for input in vmenu
            # The user can cycle through by running the command multiple times
            return True
            
        except Exception as e:
            self.env['runtime']['outputManager'].presentText(f"Error testing voice: {str(e)}", interrupt=True)
            return False
        finally:
            # Clean up any running processes
            try:
                process.terminate()
            except:
                pass
    
    def setCallback(self, callback):
        pass