#!/usr/bin/env python3

import subprocess

class command():
    def __init__(self):
        pass
    
    def initialize(self, environment):
        self.env = environment
        self.testMessage = "This is a voice preview. The quick brown fox jumps over the lazy dog."
    
    def shutdown(self):
        pass
    
    def getDescription(self):
        return "Cycle through available voices (run multiple times to browse)"
    
    def run(self):
        # Get available modules
        modules = self.getSpeechdModules()
        if not modules:
            self.env['runtime']['outputManager'].presentText("No speech-dispatcher modules found", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Error')
            return
        
        # Get stored indexes or initialize
        moduleIndex = self.env['commandBuffer'].get('voicePreviewModuleIndex', 0)
        voiceIndex = self.env['commandBuffer'].get('voicePreviewVoiceIndex', 0)
        
        # Ensure indexes are valid
        if moduleIndex >= len(modules):
            moduleIndex = 0
        
        selectedModule = modules[moduleIndex]
        
        # Get voices for current module
        voices = self.getModuleVoices(selectedModule)
        if not voices:
            self.env['runtime']['outputManager'].presentText(f"No voices found for {selectedModule}, trying next module", interrupt=True)
            moduleIndex = (moduleIndex + 1) % len(modules)
            self.env['commandBuffer']['voicePreviewModuleIndex'] = moduleIndex
            self.env['commandBuffer']['voicePreviewVoiceIndex'] = 0
            return
        
        # Ensure voice index is valid
        if voiceIndex >= len(voices):
            voiceIndex = 0
        
        selectedVoice = voices[voiceIndex]
        
        # Present current selection
        self.env['runtime']['outputManager'].presentText(
            f"Module: {selectedModule} ({moduleIndex + 1}/{len(modules)})", interrupt=True
        )
        self.env['runtime']['outputManager'].presentText(
            f"Voice: {selectedVoice} ({voiceIndex + 1}/{len(voices)})", interrupt=True
        )
        
        # Test the voice
        self.env['runtime']['outputManager'].presentText("Testing voice...", interrupt=True)
        if self.previewVoice(selectedModule, selectedVoice):
            self.env['runtime']['outputManager'].presentText("Voice test completed", interrupt=True)
            self.env['runtime']['outputManager'].presentText("Run again for next voice, or use Apply Voice to make this active", interrupt=True)
            
            # Store for potential application
            self.env['commandBuffer']['lastTestedModule'] = selectedModule
            self.env['commandBuffer']['lastTestedVoice'] = selectedVoice
            
            self.env['runtime']['outputManager'].playSound('Accept')
        else:
            self.env['runtime']['outputManager'].presentText("Voice test failed", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Error')
        
        # Advance to next voice for next run
        voiceIndex += 1
        if voiceIndex >= len(voices):
            voiceIndex = 0
            moduleIndex = (moduleIndex + 1) % len(modules)
        
        # Store indexes for next run
        self.env['commandBuffer']['voicePreviewModuleIndex'] = moduleIndex
        self.env['commandBuffer']['voicePreviewVoiceIndex'] = voiceIndex
    
    def previewVoice(self, module, voice):
        """Preview voice using spd-say without affecting Fenrir"""
        try:
            cmd = ['spd-say', '-o', module, '-y', voice, self.testMessage]
            result = subprocess.run(cmd, timeout=15)
            return result.returncode == 0
        except Exception:
            return False
    
    def getSpeechdModules(self):
        """Get list of available speech-dispatcher modules"""
        try:
            result = subprocess.run(['spd-say', '-O'], capture_output=True, text=True, timeout=10)
            if result.returncode == 0:
                lines = result.stdout.strip().split('\n')
                return [line.strip() for line in lines[1:] if line.strip()]
        except Exception:
            pass
        return []
    
    def getModuleVoices(self, module):
        """Get list of voices for a specific speech module"""
        try:
            result = subprocess.run(['spd-say', '-o', module, '-L'], capture_output=True, text=True, timeout=10)
            if result.returncode == 0:
                lines = result.stdout.strip().split('\n')
                voices = []
                for line in lines[1:]:
                    if not line.strip():
                        continue
                    if module.lower() == 'espeak-ng':
                        voice = self.processEspeakVoice(line)
                        if voice:
                            voices.append(voice)
                    else:
                        voices.append(line.strip())
                return voices
        except Exception:
            pass
        return []
    
    def processEspeakVoice(self, voiceLine):
        """Process espeak-ng voice line format"""
        parts = [p for p in voiceLine.split() if p]
        if len(parts) < 2:
            return None
        langCode = parts[-2].lower()
        variant = parts[-1].lower()
        return f"{langCode}+{variant}" if variant and variant != 'none' else langCode
    
    def setCallback(self, callback):
        pass